﻿using System;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.Data;
using Microsoft.VisualStudio;
using System.Diagnostics;
using System.Globalization;
using Microsoft.VisualStudio.TextManager.Interop;
using System.Runtime.InteropServices;

namespace MySql.Data.VisualStudio.Editors
{
    [ComVisible(true)]
    [Guid("FBAA9B3C-8DE2-46fb-A2D2-E3D39A816F4F")]
    public class CodeEditorFactory : IVsEditorFactory
    {
        private ServiceProvider vsServiceProvider;

        public CodeEditorFactory()
        {
        }

        #region IVsEditorFactory Members

        int IVsEditorFactory.Close()
        {
            return VSConstants.S_OK;
        }

        int IVsEditorFactory.CreateEditorInstance(uint grfCreateDoc, string pszMkDocument, 
            string pszPhysicalView, IVsHierarchy pvHier, uint itemid, IntPtr punkDocDataExisting, 
            out IntPtr ppunkDocView, out IntPtr ppunkDocData, out string pbstrEditorCaption, 
            out Guid pguidCmdUI, out int pgrfCDW)
        {
            Trace.WriteLine(string.Format(CultureInfo.CurrentCulture, "Entering {0} CreateEditorInstace()", this.ToString()));

            // Initialize to null
            ppunkDocView = IntPtr.Zero;
            ppunkDocData = IntPtr.Zero;
            pguidCmdUI = Guid.Empty;
            pgrfCDW = 0;
            pbstrEditorCaption = null;

            // Validate inputs
            if ((grfCreateDoc & (VSConstants.CEF_OPENFILE | VSConstants.CEF_SILENT)) == 0)
            {
                Debug.Assert(false, "Only Open or Silent is valid");
                return VSConstants.E_INVALIDARG;
            }

            // punkDocDataExisting is *not* null which means the file *is* already open. 
            // We need to verify that the open document is in fact a TextBuffer. If not
            // then we need to return the special error code VS_E_INCOMPATIBLEDOCDATA which
            // causes the user to be prompted to close the open file. If the user closes the
            // file then we will be called again with punkDocDataExisting as null.

            // QI existing buffer for text lines
            Object dataObject = Marshal.GetObjectForIUnknown(punkDocDataExisting);
            IVsTextBufferProvider provider = dataObject as IVsTextBufferProvider;
            IVsTextLines lines;
            provider.GetTextBuffer(out lines);

            ppunkDocData = punkDocDataExisting;
            Marshal.AddRef(ppunkDocData);

            ppunkDocView = CreateCodeEditor(lines);

            return VSConstants.S_OK;
        }

        private IntPtr CreateCodeEditor(IVsTextLines textLines)
        {
            Type codeWindowType = typeof(IVsCodeWindow);
            Guid riid = codeWindowType.GUID;
            Guid clsid = typeof(VsCodeWindowClass).GUID;
            IVsCodeWindow window = (IVsCodeWindow)MySqlDataProviderPackage.Instance.CreateInstance(
                ref clsid, ref riid, codeWindowType);
            window.SetBuffer(textLines);
            window.SetBaseEditorCaption(null);
//            window.GetEditorCaption(
  //          NativeMethods.ThrowOnFailure(window.SetBuffer(textLines));
    //        NativeMethods.ThrowOnFailure(window.SetBaseEditorCaption(null));
      //      NativeMethods.ThrowOnFailure(
        //        window.GetEditorCaption(READONLYSTATUS.ROSTATUS_Unknown, out editorCaption));
          //  cmdUI = NativeMethods.GUID_TextEditorFactory;

            return Marshal.GetIUnknownForObject(window);

/*            Guid clsidTextBuffer = typeof(VsTextBufferClass).GUID;
            Guid iidTextBuffer = VSConstants.IID_IUnknown;
            TextBuffer = (IVsTextBuffer)MySqlDataProviderPackage.Instance.CreateInstance(
                                 ref clsidTextBuffer,
                                 ref iidTextBuffer,
                                 typeof(object));

            // first we need to site our buffer
            IObjectWithSite ows = (IObjectWithSite)TextBuffer;
            ows.SetSite(MySqlDataProviderPackage.Instance);

            // then we need to tell our buffer not to attempt to autodetect the
            // language settings
            IVsUserData userData = TextBuffer as IVsUserData;
            Guid g = EditorFactory.GuidVSBufferDetectLangSid;
            int result = userData.SetData(ref g, false);

            Guid clsidCodeWindow = typeof(VsCodeWindowClass).GUID;
            Guid iidCodeWindow = typeof(IVsCodeWindow).GUID;
            IVsCodeWindow pCodeWindow = (IVsCodeWindow)MySqlDataProviderPackage.Instance.CreateInstance(
                   ref clsidCodeWindow,
                   ref iidCodeWindow,
                   typeof(IVsCodeWindow));
            if (pCodeWindow == null)
                throw new Exception("Failed to create core editor");

            // Give the text buffer to the code window.                    
            // We are giving up ownership of the text buffer!                    
            pCodeWindow.SetBuffer((IVsTextLines)TextBuffer);

            CodeWindow = pCodeWindow;*/
        }

        int IVsEditorFactory.MapLogicalView(ref Guid rguidLogicalView, out string pbstrPhysicalView)
        {
            pbstrPhysicalView = null;

            if (VSConstants.LOGVIEWID_Designer == rguidLogicalView ||
                VSConstants.LOGVIEWID_Primary == rguidLogicalView)
            {
                // Every EditorFactory must have one physical view identified by the null string.
                // This should be the default view for the EditorFactory.
                // We only support 1 physical view (our designer view), so return null as its
                // PhysicalView string.
                pbstrPhysicalView = null;
                return VSConstants.S_OK;
            }

            // We only support LOGVIEWID_Designer and LOGVIEWID_Primary
            return VSConstants.E_NOTIMPL;
        }

        int IVsEditorFactory.SetSite(Microsoft.VisualStudio.OLE.Interop.IServiceProvider psp)
        {
            vsServiceProvider = new ServiceProvider(psp);
            return VSConstants.S_OK;
        }

        #endregion

    }
}
